@@ -1,11 +1,11 @@
 /*
     __ _____ _____ _____
  __|  |   __|     |   | |  JSON for Modern C++ (test suite)
-|  |  |__   |  |  | | | |  version 2.1.1
+|  |  |__   |  |  | | | |  version 3.1.2
 |_____|_____|_____|_|___|  https://github.com/nlohmann/json
 
 Licensed under the MIT License <http://opensource.org/licenses/MIT>.
-Copyright (c) 2013-2017 Niels Lohmann <http://nlohmann.me>.
+Copyright (c) 2013-2018 Niels Lohmann <http://nlohmann.me>.
 
 Permission is hereby  granted, free of charge, to any  person obtaining a copy
 of this software and associated  documentation files (the "Software"), to deal
@@ -28,9 +28,11 @@ SOFTWARE.
 
 #include "catch.hpp"
 
-#include "json.hpp"
+#include <nlohmann/json.hpp>
 using nlohmann::json;
 
+#include <fstream>
+
 TEST_CASE("JSON patch")
 {
     SECTION("examples from RFC 6902")
@@ -75,8 +77,9 @@ TEST_CASE("JSON patch")
             json doc2 = R"({ "q": { "bar": 2 } })"_json;
 
             // because "a" does not exist.
-            CHECK_THROWS_AS(doc2.patch(patch), std::out_of_range);
-            CHECK_THROWS_WITH(doc2.patch(patch), "key 'a' not found");
+            CHECK_THROWS_AS(doc2.patch(patch), json::out_of_range&);
+            CHECK_THROWS_WITH(doc2.patch(patch),
+                              "[json.exception.out_of_range.403] key 'a' not found");
         }
 
         SECTION("4.2 remove")
@@ -336,8 +339,8 @@ TEST_CASE("JSON patch")
                 )"_json;
 
             // check that evaluation throws
-            CHECK_THROWS_AS(doc.patch(patch), std::domain_error);
-            CHECK_THROWS_WITH(doc.patch(patch), "unsuccessful: " + patch[0].dump());
+            CHECK_THROWS_AS(doc.patch(patch), json::other_error&);
+            CHECK_THROWS_WITH(doc.patch(patch), "[json.exception.other_error.501] unsuccessful: " + patch[0].dump());
         }
 
         SECTION("A.10. Adding a Nested Member Object")
@@ -420,8 +423,9 @@ TEST_CASE("JSON patch")
             // references neither the root of the document, nor a member of
             // an existing object, nor a member of an existing array.
 
-            CHECK_THROWS_AS(doc.patch(patch), std::out_of_range);
-            CHECK_THROWS_WITH(doc.patch(patch), "key 'baz' not found");
+            CHECK_THROWS_AS(doc.patch(patch), json::out_of_range&);
+            CHECK_THROWS_WITH(doc.patch(patch),
+                              "[json.exception.out_of_range.403] key 'baz' not found");
         }
 
         // A.13. Invalid JSON Patch Document
@@ -476,8 +480,8 @@ TEST_CASE("JSON patch")
                 )"_json;
 
             // check that evaluation throws
-            CHECK_THROWS_AS(doc.patch(patch), std::domain_error);
-            CHECK_THROWS_WITH(doc.patch(patch), "unsuccessful: " + patch[0].dump());
+            CHECK_THROWS_AS(doc.patch(patch), json::other_error&);
+            CHECK_THROWS_WITH(doc.patch(patch), "[json.exception.other_error.501] unsuccessful: " + patch[0].dump());
         }
 
         SECTION("A.16. Adding an Array Value")
@@ -666,40 +670,45 @@ TEST_CASE("JSON patch")
             {
                 json j;
                 json patch = {{"op", "add"}, {"path", ""}, {"value", 1}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "JSON patch must be an array of objects");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.104] parse error: JSON patch must be an array of objects");
             }
 
             SECTION("not an array of objects")
             {
                 json j;
                 json patch = {"op", "add", "path", "", "value", 1};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "JSON patch must be an array of objects");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.104] parse error: JSON patch must be an array of objects");
             }
 
             SECTION("missing 'op'")
             {
                 json j;
                 json patch = {{{"foo", "bar"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation must have member 'op'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation must have member 'op'");
             }
 
             SECTION("non-string 'op'")
             {
                 json j;
                 json patch = {{{"op", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation must have string member 'op'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation must have string member 'op'");
             }
 
             SECTION("invalid operation")
             {
                 json j;
                 json patch = {{{"op", "foo"}, {"path", ""}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation value 'foo' is invalid");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation value 'foo' is invalid");
             }
         }
 
@@ -709,32 +718,36 @@ TEST_CASE("JSON patch")
             {
                 json j;
                 json patch = {{{"op", "add"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'add' must have member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'add' must have member 'path'");
             }
 
             SECTION("non-string 'path'")
             {
                 json j;
                 json patch = {{{"op", "add"}, {"path", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'add' must have string member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'add' must have string member 'path'");
             }
 
             SECTION("missing 'value'")
             {
                 json j;
                 json patch = {{{"op", "add"}, {"path", ""}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'add' must have member 'value'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'add' must have member 'value'");
             }
 
             SECTION("invalid array index")
             {
                 json j = {1, 2};
                 json patch = {{{"op", "add"}, {"path", "/4"}, {"value", 4}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "array index 4 is out of range");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.401] array index 4 is out of range");
             }
         }
 
@@ -744,40 +757,45 @@ TEST_CASE("JSON patch")
             {
                 json j;
                 json patch = {{{"op", "remove"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'remove' must have member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'remove' must have member 'path'");
             }
 
             SECTION("non-string 'path'")
             {
                 json j;
                 json patch = {{{"op", "remove"}, {"path", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'remove' must have string member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'remove' must have string member 'path'");
             }
 
             SECTION("nonexisting target location (array)")
             {
                 json j = {1, 2, 3};
                 json patch = {{{"op", "remove"}, {"path", "/17"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "array index 17 is out of range");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.401] array index 17 is out of range");
             }
 
             SECTION("nonexisting target location (object)")
             {
                 json j = {{"foo", 1}, {"bar", 2}};
                 json patch = {{{"op", "remove"}, {"path", "/baz"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "key 'baz' not found");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.403] key 'baz' not found");
             }
 
             SECTION("root element as target location")
             {
                 json j = "string";
                 json patch = {{{"op", "remove"}, {"path", ""}}};
-                CHECK_THROWS_AS(j.patch(patch), std::domain_error);
-                CHECK_THROWS_WITH(j.patch(patch), "JSON pointer has no parent");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.405] JSON pointer has no parent");
             }
         }
 
@@ -787,40 +805,45 @@ TEST_CASE("JSON patch")
             {
                 json j;
                 json patch = {{{"op", "replace"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'replace' must have member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'replace' must have member 'path'");
             }
 
             SECTION("non-string 'path'")
             {
                 json j;
                 json patch = {{{"op", "replace"}, {"path", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'replace' must have string member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'replace' must have string member 'path'");
             }
 
             SECTION("missing 'value'")
             {
                 json j;
                 json patch = {{{"op", "replace"}, {"path", ""}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'replace' must have member 'value'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'replace' must have member 'value'");
             }
 
             SECTION("nonexisting target location (array)")
             {
                 json j = {1, 2, 3};
                 json patch = {{{"op", "replace"}, {"path", "/17"}, {"value", 19}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "array index 17 is out of range");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.401] array index 17 is out of range");
             }
 
             SECTION("nonexisting target location (object)")
             {
                 json j = {{"foo", 1}, {"bar", 2}};
                 json patch = {{{"op", "replace"}, {"path", "/baz"}, {"value", 3}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "key 'baz' not found");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.403] key 'baz' not found");
             }
         }
 
@@ -830,48 +853,54 @@ TEST_CASE("JSON patch")
             {
                 json j;
                 json patch = {{{"op", "move"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'move' must have member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'move' must have member 'path'");
             }
 
             SECTION("non-string 'path'")
             {
                 json j;
                 json patch = {{{"op", "move"}, {"path", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'move' must have string member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'move' must have string member 'path'");
             }
 
             SECTION("missing 'from'")
             {
                 json j;
                 json patch = {{{"op", "move"}, {"path", ""}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'move' must have member 'from'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'move' must have member 'from'");
             }
 
             SECTION("non-string 'from'")
             {
                 json j;
                 json patch = {{{"op", "move"}, {"path", ""}, {"from", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'move' must have string member 'from'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'move' must have string member 'from'");
             }
 
             SECTION("nonexisting from location (array)")
             {
                 json j = {1, 2, 3};
                 json patch = {{{"op", "move"}, {"path", "/0"}, {"from", "/5"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "array index 5 is out of range");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.401] array index 5 is out of range");
             }
 
             SECTION("nonexisting from location (object)")
             {
                 json j = {{"foo", 1}, {"bar", 2}};
                 json patch = {{{"op", "move"}, {"path", "/baz"}, {"from", "/baz"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "key 'baz' not found");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.403] key 'baz' not found");
             }
         }
 
@@ -881,48 +910,54 @@ TEST_CASE("JSON patch")
             {
                 json j;
                 json patch = {{{"op", "copy"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'copy' must have member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'copy' must have member 'path'");
             }
 
             SECTION("non-string 'path'")
             {
                 json j;
                 json patch = {{{"op", "copy"}, {"path", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'copy' must have string member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'copy' must have string member 'path'");
             }
 
             SECTION("missing 'from'")
             {
                 json j;
                 json patch = {{{"op", "copy"}, {"path", ""}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'copy' must have member 'from'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'copy' must have member 'from'");
             }
 
             SECTION("non-string 'from'")
             {
                 json j;
                 json patch = {{{"op", "copy"}, {"path", ""}, {"from", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'copy' must have string member 'from'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'copy' must have string member 'from'");
             }
 
             SECTION("nonexisting from location (array)")
             {
                 json j = {1, 2, 3};
                 json patch = {{{"op", "copy"}, {"path", "/0"}, {"from", "/5"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "array index 5 is out of range");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.401] array index 5 is out of range");
             }
 
             SECTION("nonexisting from location (object)")
             {
                 json j = {{"foo", 1}, {"bar", 2}};
                 json patch = {{{"op", "copy"}, {"path", "/fob"}, {"from", "/baz"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::out_of_range);
-                CHECK_THROWS_WITH(j.patch(patch), "key 'baz' not found");
+                CHECK_THROWS_AS(j.patch(patch), json::out_of_range&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.out_of_range.403] key 'baz' not found");
             }
         }
 
@@ -932,24 +967,27 @@ TEST_CASE("JSON patch")
             {
                 json j;
                 json patch = {{{"op", "test"}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'test' must have member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'test' must have member 'path'");
             }
 
             SECTION("non-string 'path'")
             {
                 json j;
                 json patch = {{{"op", "test"}, {"path", 1}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'test' must have string member 'path'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'test' must have string member 'path'");
             }
 
             SECTION("missing 'value'")
             {
                 json j;
                 json patch = {{{"op", "test"}, {"path", ""}}};
-                CHECK_THROWS_AS(j.patch(patch), std::invalid_argument);
-                CHECK_THROWS_WITH(j.patch(patch), "operation 'test' must have member 'value'");
+                CHECK_THROWS_AS(j.patch(patch), json::parse_error&);
+                CHECK_THROWS_WITH(j.patch(patch),
+                                  "[json.exception.parse_error.105] parse error: operation 'test' must have member 'value'");
             }
         }
     }
@@ -1141,8 +1179,8 @@ TEST_CASE("JSON patch")
                 )"_json;
 
                 // the test will fail
-                CHECK_THROWS_AS(doc.patch(patch), std::domain_error);
-                CHECK_THROWS_WITH(doc.patch(patch), "unsuccessful: " + patch[0].dump());
+                CHECK_THROWS_AS(doc.patch(patch), json::other_error&);
+                CHECK_THROWS_WITH(doc.patch(patch), "[json.exception.other_error.501] unsuccessful: " + patch[0].dump());
             }
         }
     }
@@ -1214,4 +1252,42 @@ TEST_CASE("JSON patch")
                               R"( [{"op": "test", "path": "/foo", "value": "bar"}] )"_json));
         }
     }
+
+    SECTION("Tests from github.com/json-patch/json-patch-tests")
+    {
+        for (auto filename :
+                {"test/data/json-patch-tests/spec_tests.json",
+                 "test/data/json-patch-tests/tests.json"
+                })
+        {
+            CAPTURE(filename);
+            std::ifstream f(filename);
+            json suite = json::parse(f);
+
+            for (const auto& test : suite)
+            {
+                CAPTURE(test.value("comment", ""))
+
+                // skip tests marked as disabled
+                if (test.value("disabled", false))
+                {
+                    continue;
+                }
+
+                const auto& doc = test["doc"];
+                const auto& patch = test["patch"];
+
+                if (test.count("error") == 0)
+                {
+                    // if an expected value is given, use it; use doc otherwise
+                    const auto& expected = test.value("expected", doc);
+                    CHECK(doc.patch(patch) == expected);
+                }
+                else
+                {
+                    CHECK_THROWS(doc.patch(patch));
+                }
+            }
+        }
+    }
 }
