@@ -1,11 +1,11 @@
 /*
     __ _____ _____ _____
  __|  |   __|     |   | |  JSON for Modern C++ (test suite)
-|  |  |__   |  |  | | | |  version 2.1.1
+|  |  |__   |  |  | | | |  version 3.1.2
 |_____|_____|_____|_|___|  https://github.com/nlohmann/json
 
 Licensed under the MIT License <http://opensource.org/licenses/MIT>.
-Copyright (c) 2013-2017 Niels Lohmann <http://nlohmann.me>.
+Copyright (c) 2013-2018 Niels Lohmann <http://nlohmann.me>.
 
 Permission is hereby  granted, free of charge, to any  person obtaining a copy
 of this software and associated  documentation files (the "Software"), to deal
@@ -28,7 +28,7 @@ SOFTWARE.
 
 #include "catch.hpp"
 
-#include "json.hpp"
+#include <nlohmann/json.hpp>
 using nlohmann::json;
 
 TEST_CASE("modifiers")
@@ -38,17 +38,21 @@ TEST_CASE("modifiers")
         SECTION("boolean")
         {
             json j = true;
+            json k = j;
 
             j.clear();
             CHECK(j == json(json::value_t::boolean));
+            CHECK(j == json(k.type()));
         }
 
         SECTION("string")
         {
             json j = "hello world";
+            json k = j;
 
             j.clear();
             CHECK(j == json(json::value_t::string));
+            CHECK(j == json(k.type()));
         }
 
         SECTION("array")
@@ -56,19 +60,23 @@ TEST_CASE("modifiers")
             SECTION("empty array")
             {
                 json j = json::array();
+                json k = j;
 
                 j.clear();
                 CHECK(j.empty());
                 CHECK(j == json(json::value_t::array));
+                CHECK(j == json(k.type()));
             }
 
             SECTION("filled array")
             {
                 json j = {1, 2, 3};
+                json k = j;
 
                 j.clear();
                 CHECK(j.empty());
                 CHECK(j == json(json::value_t::array));
+                CHECK(j == json(k.type()));
             }
         }
 
@@ -77,52 +85,64 @@ TEST_CASE("modifiers")
             SECTION("empty object")
             {
                 json j = json::object();
+                json k = j;
 
                 j.clear();
                 CHECK(j.empty());
                 CHECK(j == json(json::value_t::object));
+                CHECK(j == json(k.type()));
             }
 
             SECTION("filled object")
             {
                 json j = {{"one", 1}, {"two", 2}, {"three", 3}};
+                json k = j;
 
                 j.clear();
                 CHECK(j.empty());
                 CHECK(j == json(json::value_t::object));
+                CHECK(j == json(k.type()));
             }
         }
 
         SECTION("number (integer)")
         {
             json j = 23;
+            json k = j;
 
             j.clear();
             CHECK(j == json(json::value_t::number_integer));
+            CHECK(j == json(k.type()));
         }
 
         SECTION("number (unsigned)")
         {
             json j = 23u;
+            json k = j;
 
             j.clear();
             CHECK(j == json(json::value_t::number_integer));
+            CHECK(j == json(k.type()));
         }
 
         SECTION("number (float)")
         {
             json j = 23.42;
+            json k = j;
 
             j.clear();
             CHECK(j == json(json::value_t::number_float));
+            CHECK(j == json(k.type()));
         }
 
         SECTION("null")
         {
             json j = nullptr;
+            json k = j;
 
             j.clear();
             CHECK(j == json(json::value_t::null));
+            CHECK(j == json(k.type()));
         }
     }
 
@@ -152,8 +172,8 @@ TEST_CASE("modifiers")
                 SECTION("other type")
                 {
                     json j = 1;
-                    CHECK_THROWS_AS(j.push_back("Hello"), std::domain_error);
-                    CHECK_THROWS_WITH(j.push_back("Hello"), "cannot use push_back() with number");
+                    CHECK_THROWS_AS(j.push_back("Hello"), json::type_error&);
+                    CHECK_THROWS_WITH(j.push_back("Hello"), "[json.exception.type_error.308] cannot use push_back() with number");
                 }
             }
 
@@ -182,8 +202,8 @@ TEST_CASE("modifiers")
                 {
                     json j = 1;
                     json k("Hello");
-                    CHECK_THROWS_AS(j.push_back(k), std::domain_error);
-                    CHECK_THROWS_WITH(j.push_back(k), "cannot use push_back() with number");
+                    CHECK_THROWS_AS(j.push_back(k), json::type_error&);
+                    CHECK_THROWS_WITH(j.push_back(k), "[json.exception.type_error.308] cannot use push_back() with number");
                 }
             }
         }
@@ -215,9 +235,9 @@ TEST_CASE("modifiers")
             {
                 json j = 1;
                 json k("Hello");
-                CHECK_THROWS_AS(j.push_back(json::object_t::value_type({"one", 1})), std::domain_error);
+                CHECK_THROWS_AS(j.push_back(json::object_t::value_type({"one", 1})), json::type_error&);
                 CHECK_THROWS_WITH(j.push_back(json::object_t::value_type({"one", 1})),
-                                  "cannot use push_back() with number");
+                                  "[json.exception.type_error.308] cannot use push_back() with number");
             }
         }
 
@@ -251,9 +271,13 @@ TEST_CASE("modifiers")
                 j.push_back({"key2", "bar"});
                 CHECK(j == json({{"key1", 1}, {"key2", "bar"}}));
 
-                json k = {{"key1", 1}};
-                CHECK_THROWS_AS(k.push_back({1, 2, 3, 4}), std::domain_error);
-                CHECK_THROWS_WITH(k.push_back({1, 2, 3, 4}), "cannot use push_back() with object");
+                // invalid values (no string/val pair)
+                CHECK_THROWS_AS(j.push_back({1}), json::type_error&);
+                CHECK_THROWS_WITH(j.push_back({1}), "[json.exception.type_error.308] cannot use push_back() with object");
+                CHECK_THROWS_AS(j.push_back({1, 2}), json::type_error&);
+                CHECK_THROWS_WITH(j.push_back({1, 2}), "[json.exception.type_error.308] cannot use push_back() with object");
+                CHECK_THROWS_AS(j.push_back({1, 2, 3, 4}), json::type_error&);
+                CHECK_THROWS_WITH(j.push_back({1, 2, 3, 4}), "[json.exception.type_error.308] cannot use push_back() with object");
             }
         }
     }
@@ -291,8 +315,9 @@ TEST_CASE("modifiers")
         SECTION("other type")
         {
             json j = 1;
-            CHECK_THROWS_AS(j.emplace_back("Hello"), std::domain_error);
-            CHECK_THROWS_WITH(j.emplace_back("Hello"), "cannot use emplace_back() with number");
+            CHECK_THROWS_AS(j.emplace_back("Hello"), json::type_error&);
+            CHECK_THROWS_WITH(j.emplace_back("Hello"),
+                              "[json.exception.type_error.311] cannot use emplace_back() with number");
         }
     }
 
@@ -350,8 +375,9 @@ TEST_CASE("modifiers")
         SECTION("other type")
         {
             json j = 1;
-            CHECK_THROWS_AS(j.emplace("foo", "bar"), std::domain_error);
-            CHECK_THROWS_WITH(j.emplace("foo", "bar"), "cannot use emplace() with number");
+            CHECK_THROWS_AS(j.emplace("foo", "bar"), json::type_error&);
+            CHECK_THROWS_WITH(j.emplace("foo", "bar"),
+                              "[json.exception.type_error.311] cannot use emplace() with number");
         }
     }
 
@@ -381,8 +407,8 @@ TEST_CASE("modifiers")
                 SECTION("other type")
                 {
                     json j = 1;
-                    CHECK_THROWS_AS(j += "Hello", std::domain_error);
-                    CHECK_THROWS_WITH(j += "Hello", "cannot use push_back() with number");
+                    CHECK_THROWS_AS(j += "Hello", json::type_error&);
+                    CHECK_THROWS_WITH(j += "Hello", "[json.exception.type_error.308] cannot use push_back() with number");
                 }
             }
 
@@ -411,8 +437,8 @@ TEST_CASE("modifiers")
                 {
                     json j = 1;
                     json k("Hello");
-                    CHECK_THROWS_AS(j += k, std::domain_error);
-                    CHECK_THROWS_WITH(j += k, "cannot use push_back() with number");
+                    CHECK_THROWS_AS(j += k, json::type_error&);
+                    CHECK_THROWS_WITH(j += k, "[json.exception.type_error.308] cannot use push_back() with number");
                 }
             }
         }
@@ -444,9 +470,9 @@ TEST_CASE("modifiers")
             {
                 json j = 1;
                 json k("Hello");
-                CHECK_THROWS_AS(j += json::object_t::value_type({"one", 1}), std::domain_error);
+                CHECK_THROWS_AS(j += json::object_t::value_type({"one", 1}), json::type_error&);
                 CHECK_THROWS_WITH(j += json::object_t::value_type({"one", 1}),
-                                  "cannot use push_back() with number");
+                                  "[json.exception.type_error.308] cannot use push_back() with number");
             }
         }
 
@@ -481,13 +507,13 @@ TEST_CASE("modifiers")
                 CHECK(j == json({{"key1", 1}, {"key2", "bar"}}));
 
                 json k = {{"key1", 1}};
-                CHECK_THROWS_AS((k += {1, 2, 3, 4}), std::domain_error);
-                CHECK_THROWS_WITH((k += {1, 2, 3, 4}), "cannot use push_back() with object");
+                CHECK_THROWS_AS((k += {1, 2, 3, 4}), json::type_error&);
+                CHECK_THROWS_WITH((k += {1, 2, 3, 4}), "[json.exception.type_error.308] cannot use push_back() with object");
             }
         }
     }
 
-    SECTION("insert")
+    SECTION("insert()")
     {
         json j_array = {1, 2, 3, 4};
         json j_value = 5;
@@ -583,15 +609,16 @@ TEST_CASE("modifiers")
 
             SECTION("insert nothing (count = 0)")
             {
-                auto pos = j_array.end();
                 auto it = j_array.insert(j_array.end(), 0, 5);
                 CHECK(j_array.size() == 4);
-                CHECK(it == pos);
+                // the returned iterator points to the first inserted element;
+                // there were 4 elements, so it should point to the 5th
+                CHECK(it == j_array.begin() + 4);
                 CHECK(j_array == json({1, 2, 3, 4}));
             }
         }
 
-        SECTION("range")
+        SECTION("range for array")
         {
             json j_other_array = {"first", "second"};
 
@@ -616,14 +643,49 @@ TEST_CASE("modifiers")
             {
                 json j_other_array2 = {"first", "second"};
 
-                CHECK_THROWS_AS(j_array.insert(j_array.end(), j_array.begin(), j_array.end()), std::domain_error);
+                CHECK_THROWS_AS(j_array.insert(j_array.end(), j_array.begin(), j_array.end()),
+                                json::invalid_iterator&);
                 CHECK_THROWS_AS(j_array.insert(j_array.end(), j_other_array.begin(), j_other_array2.end()),
-                                std::domain_error);
+                                json::invalid_iterator&);
 
                 CHECK_THROWS_WITH(j_array.insert(j_array.end(), j_array.begin(), j_array.end()),
-                                  "passed iterators may not belong to container");
+                                  "[json.exception.invalid_iterator.211] passed iterators may not belong to container");
                 CHECK_THROWS_WITH(j_array.insert(j_array.end(), j_other_array.begin(), j_other_array2.end()),
-                                  "iterators do not fit");
+                                  "[json.exception.invalid_iterator.210] iterators do not fit");
+            }
+        }
+
+        SECTION("range for object")
+        {
+            json j_object1 = {{"one", "eins"}, {"two", "zwei"}};
+            json j_object2 = {{"eleven", "elf"}, {"seventeen", "siebzehn"}};
+
+            SECTION("proper usage")
+            {
+                j_object1.insert(j_object2.begin(), j_object2.end());
+                CHECK(j_object1.size() == 4);
+            }
+
+            SECTION("empty range")
+            {
+                j_object1.insert(j_object2.begin(), j_object2.begin());
+                CHECK(j_object1.size() == 2);
+            }
+
+            SECTION("invalid iterators")
+            {
+                json j_other_array2 = {"first", "second"};
+
+                CHECK_THROWS_AS(j_array.insert(j_object2.begin(), j_object2.end()), json::type_error&);
+                CHECK_THROWS_AS(j_object1.insert(j_object1.begin(), j_object2.end()), json::invalid_iterator&);
+                CHECK_THROWS_AS(j_object1.insert(j_array.begin(), j_array.end()), json::invalid_iterator&);
+
+                CHECK_THROWS_WITH(j_array.insert(j_object2.begin(), j_object2.end()),
+                                  "[json.exception.type_error.309] cannot use insert() with array");
+                CHECK_THROWS_WITH(j_object1.insert(j_object1.begin(), j_object2.end()),
+                                  "[json.exception.invalid_iterator.210] iterators do not fit");
+                CHECK_THROWS_WITH(j_object1.insert(j_array.begin(), j_array.end()),
+                                  "[json.exception.invalid_iterator.202] iterators first and last must point to objects");
             }
         }
 
@@ -662,22 +724,22 @@ TEST_CASE("modifiers")
             // pass iterator to a different array
             json j_another_array = {1, 2};
             json j_yet_another_array = {"first", "second"};
-            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), 10), std::domain_error);
-            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), j_value), std::domain_error);
-            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), 10, 11), std::domain_error);
-            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), j_yet_another_array.begin(),
-                                           j_yet_another_array.end()), std::domain_error);
-            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), {1, 2, 3, 4}), std::domain_error);
+            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), 10), json::invalid_iterator&);
+            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), j_value), json::invalid_iterator&);
+            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), 10, 11), json::invalid_iterator&);
+            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), j_yet_another_array.begin(), j_yet_another_array.end()), json::invalid_iterator&);
+            CHECK_THROWS_AS(j_array.insert(j_another_array.end(), {1, 2, 3, 4}), json::invalid_iterator&);
 
-            CHECK_THROWS_WITH(j_array.insert(j_another_array.end(), 10), "iterator does not fit current value");
+            CHECK_THROWS_WITH(j_array.insert(j_another_array.end(), 10),
+                              "[json.exception.invalid_iterator.202] iterator does not fit current value");
             CHECK_THROWS_WITH(j_array.insert(j_another_array.end(), j_value),
-                              "iterator does not fit current value");
+                              "[json.exception.invalid_iterator.202] iterator does not fit current value");
             CHECK_THROWS_WITH(j_array.insert(j_another_array.end(), 10, 11),
-                              "iterator does not fit current value");
-            CHECK_THROWS_WITH(j_array.insert(j_another_array.end(), j_yet_another_array.begin(),
-                                             j_yet_another_array.end()), "iterator does not fit current value");
+                              "[json.exception.invalid_iterator.202] iterator does not fit current value");
+            CHECK_THROWS_WITH(j_array.insert(j_another_array.end(), j_yet_another_array.begin(), j_yet_another_array.end()),
+                              "[json.exception.invalid_iterator.202] iterator does not fit current value");
             CHECK_THROWS_WITH(j_array.insert(j_another_array.end(), {1, 2, 3, 4}),
-                              "iterator does not fit current value");
+                              "[json.exception.invalid_iterator.202] iterator does not fit current value");
         }
 
         SECTION("non-array type")
@@ -685,20 +747,84 @@ TEST_CASE("modifiers")
             // call insert on a non-array type
             json j_nonarray = 3;
             json j_yet_another_array = {"first", "second"};
-            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), 10), std::domain_error);
-            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), j_value), std::domain_error);
-            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), 10, 11), std::domain_error);
+            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), 10), json::type_error&);
+            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), j_value), json::type_error&);
+            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), 10, 11), json::type_error&);
             CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), j_yet_another_array.begin(),
-                                              j_yet_another_array.end()), std::domain_error);
-            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), {1, 2, 3, 4}), std::domain_error);
+                                              j_yet_another_array.end()), json::type_error&);
+            CHECK_THROWS_AS(j_nonarray.insert(j_nonarray.end(), {1, 2, 3, 4}), json::type_error&);
 
-            CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), 10), "cannot use insert() with number");
-            CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), j_value), "cannot use insert() with number");
-            CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), 10, 11), "cannot use insert() with number");
+            CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), 10), "[json.exception.type_error.309] cannot use insert() with number");
+            CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), j_value), "[json.exception.type_error.309] cannot use insert() with number");
+            CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), 10, 11), "[json.exception.type_error.309] cannot use insert() with number");
             CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), j_yet_another_array.begin(),
-                                                j_yet_another_array.end()), "cannot use insert() with number");
+                                                j_yet_another_array.end()), "[json.exception.type_error.309] cannot use insert() with number");
             CHECK_THROWS_WITH(j_nonarray.insert(j_nonarray.end(), {1, 2, 3, 4}),
-                              "cannot use insert() with number");
+                              "[json.exception.type_error.309] cannot use insert() with number");
+        }
+    }
+
+    SECTION("update()")
+    {
+        json j_object1 = {{"one", "eins"}, {"two", "zwei"}};
+        json j_object2 = {{"three", "drei"}, {"two", "zwo"}};
+        json j_array = {1, 2, 3, 4};
+
+        SECTION("const reference")
+        {
+            SECTION("proper usage")
+            {
+                j_object1.update(j_object2);
+                CHECK(j_object1 == json({{"one", "eins"}, {"two", "zwo"}, {"three", "drei"}}));
+
+                json j_null;
+                j_null.update(j_object2);
+                CHECK(j_null == j_object2);
+            }
+
+            SECTION("wrong types")
+            {
+                CHECK_THROWS_AS(j_array.update(j_object1), json::type_error&);
+                CHECK_THROWS_WITH(j_array.update(j_object1), "[json.exception.type_error.312] cannot use update() with array");
+
+                CHECK_THROWS_AS(j_object1.update(j_array), json::type_error&);
+                CHECK_THROWS_WITH(j_object1.update(j_array), "[json.exception.type_error.312] cannot use update() with array");
+            }
+        }
+
+        SECTION("iterator range")
+        {
+            SECTION("proper usage")
+            {
+                j_object1.update(j_object2.begin(), j_object2.end());
+                CHECK(j_object1 == json({{"one", "eins"}, {"two", "zwo"}, {"three", "drei"}}));
+
+                json j_null;
+                j_null.update(j_object2.begin(), j_object2.end());
+                CHECK(j_null == j_object2);
+            }
+
+            SECTION("empty range")
+            {
+                j_object1.update(j_object2.begin(), j_object2.begin());
+                CHECK(j_object1 == json({{"one", "eins"}, {"two", "zwei"}}));
+            }
+
+            SECTION("invalid iterators")
+            {
+                json j_other_array2 = {"first", "second"};
+
+                CHECK_THROWS_AS(j_array.update(j_object2.begin(), j_object2.end()), json::type_error&);
+                CHECK_THROWS_AS(j_object1.update(j_object1.begin(), j_object2.end()), json::invalid_iterator&);
+                CHECK_THROWS_AS(j_object1.update(j_array.begin(), j_array.end()), json::invalid_iterator&);
+
+                CHECK_THROWS_WITH(j_array.update(j_object2.begin(), j_object2.end()),
+                                  "[json.exception.type_error.312] cannot use update() with array");
+                CHECK_THROWS_WITH(j_object1.update(j_object1.begin(), j_object2.end()),
+                                  "[json.exception.invalid_iterator.210] iterators do not fit");
+                CHECK_THROWS_WITH(j_object1.update(j_array.begin(), j_array.end()),
+                                  "[json.exception.invalid_iterator.202] iterators first and last must point to objects");
+            }
         }
     }
 
@@ -750,8 +876,8 @@ TEST_CASE("modifiers")
                 json j = 17;
                 json::array_t a = {"foo", "bar", "baz"};
 
-                CHECK_THROWS_AS(j.swap(a), std::domain_error);
-                CHECK_THROWS_WITH(j.swap(a), "cannot use swap() with number");
+                CHECK_THROWS_AS(j.swap(a), json::type_error&);
+                CHECK_THROWS_WITH(j.swap(a), "[json.exception.type_error.310] cannot use swap() with number");
             }
         }
 
@@ -776,8 +902,8 @@ TEST_CASE("modifiers")
                 json j = 17;
                 json::object_t o = {{"cow", "Kuh"}, {"chicken", "Huhn"}};
 
-                CHECK_THROWS_AS(j.swap(o), std::domain_error);
-                CHECK_THROWS_WITH(j.swap(o), "cannot use swap() with number");
+                CHECK_THROWS_AS(j.swap(o), json::type_error&);
+                CHECK_THROWS_WITH(j.swap(o), "[json.exception.type_error.310] cannot use swap() with number");
             }
         }
 
@@ -802,8 +928,8 @@ TEST_CASE("modifiers")
                 json j = 17;
                 json::string_t s = "Hallo Welt";
 
-                CHECK_THROWS_AS(j.swap(s), std::domain_error);
-                CHECK_THROWS_WITH(j.swap(s), "cannot use swap() with number");
+                CHECK_THROWS_AS(j.swap(s), json::type_error&);
+                CHECK_THROWS_WITH(j.swap(s), "[json.exception.type_error.310] cannot use swap() with number");
             }
         }
     }
